package gssim.db.mysql;

import gssim.db.dao.data.Allocation;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

public class AllocationDAO extends AbstractDAO implements
		gssim.db.dao.AllocationDAO {

	protected AllocationDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(AllocationDAO.class);
		setRelationName("allocation");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM allocation WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM allocation WHERE (task_id = ? OR ISNULL(?))"
				+ "AND (unit_id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO allocation(task_id, unit_id, begin, end) "
				+ "VALUES(?, ?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM allocation WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE allocation SET task_id = ?, unit_id = ?, begin = ?, end = ? WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(Allocation obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(Allocation obj, ResultSet resultSet)
			throws SQLException {

		ResultSetAdapter result = (ResultSetAdapter) resultSet;

		Integer pK = result.getIntObj("PK");
		Integer taskId = result.getIntObj("task_id");
		Integer unitId = result.getIntObj("unit_id");
		Timestamp begin = result.getTimestamp("begin");
		Timestamp end = result.getTimestamp("end");

		obj.setPK(pK);
		obj.setTaskId(taskId);
		obj.setUnitId(unitId);
		obj.setBegin(begin);
		obj.setEnd(end);

	}

	@Override
	public int fillStatementWithObject(PreparedStatement pstmt,
			Allocation obj, int offset) throws SQLException {

		PreparedStatementAdapter stmt = (PreparedStatementAdapter) pstmt;

		stmt.setInt(offset + 1, obj.getTaskId());
		stmt.setInt(offset + 2, obj.getUnitId());
		stmt.setTimestamp(offset + 3, obj.getBegin());
		stmt.setTimestamp(offset + 4, obj.getEnd());

		return offset + 4;

	}

	@Override
	public List<Allocation> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<Allocation> list = new ArrayList<Allocation>();

		stmt.setInt(1, finder.getInteger("taskId"));
		stmt.setInt(2, finder.getInteger("taskId"));
		stmt.setInt(3, finder.getInteger("unitId"));
		stmt.setInt(4, finder.getInteger("unitId"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {

			Allocation obj = new Allocation();
			fillObjectFromResultSet(obj, result);
			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(Allocation obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		fillStatementWithObject(stmt, obj, 0);

		int result = stmt.executeUpdate();

		if (result == 1) {
			ResultSet gk = stmt.getGeneratedKeys();
			Integer pK = gk.next() ? gk.getInt(1) : -1;
			obj.setPK(pK);

			if (commit)
				commit();

			logSuccess("insert");

		} else
			logError("insert", result, obj);

		return result;
	}

	public int[] insertBatch(List<Allocation> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			fillStatementWithObject(stmt, objList.get(i), 0);
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {

			ResultSet gk = stmt.getGeneratedKeys();
			for (int i = 0; i < objList.size(); i++) {
				Integer PK = gk.next() ? gk.getInt(1) : -1;
				objList.get(i).setPK(PK);
			}

			logSuccess("batch insert", j);

			if (commit)
				commit();

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public Allocation select(Allocation obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getPK());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			fillObjectFromResultSet(obj, result);
			return obj;
		}

		// no allocation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(Allocation obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();

		int offset = fillStatementWithObject(stmt, obj, 0);

		stmt.setInt(offset + 1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
